﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Windows.Input
Imports System.Windows.Shapes
Imports Microsoft.Win32
Imports System.Windows.Markup
Imports System.IO.File

Partial Public Class Window1
	Public Sub New()
		MyBase.New()

		Me.InitializeComponent()

		' Insert code required on object creation below this point.
    End Sub

    ' Remember if we have unsaved changes.
    Private m_IsDirty As Boolean = False

    ' The name of the currently open file.
    Private m_FileName As String = Nothing

    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles MyBase.Loaded
        ' Create the Close command here
        ' just to show how it's done in code-behind.
        Dim close_binding As New CommandBinding( _
            ApplicationCommands.Close, _
            AddressOf DocumentClose, _
            AddressOf DocumentCloseAllowed)
        Me.CommandBindings.Add(close_binding)
    End Sub

    ' Return True if we can perform the New action.
    Private Sub DocumentNewAllowed(ByVal sender As Object, ByVal e As CanExecuteRoutedEventArgs)
        e.CanExecute = (Not m_IsDirty)
    End Sub

    ' Perform the New action.
    Private Sub DocumentNew(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' Remove any existing controls on the Canvas.
        cvsDrawing.Children.Clear()

        ' Display the canvas.
        borDrawing.Visibility = Windows.Visibility.Visible
        m_FileName = Nothing
    End Sub

    ' Return True if we can perform the Open action.
    Private Sub DocumentOpenAllowed(ByVal sender As Object, ByVal e As CanExecuteRoutedEventArgs)
        e.CanExecute = (Not m_IsDirty)
    End Sub

    ' Perform the Open action.
    Private Sub DocumentOpen(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        Dim dlg As New OpenFileDialog()
        dlg.DefaultExt = ".rects"
        dlg.Filter = "Rectangle Files (.rects)|*.rects|All Files (*.*)|*.*"

        If dlg.ShowDialog() = True Then
            ' Open the file.
            Dim reader As New XamlReader()
            Dim fs As FileStream = OpenRead(dlg.FileName)
            Try
                ' Read the Canvas stored in the file.
                Dim new_canvas As Canvas = XamlReader.Load(fs)

                ' Display the new Canvas.
                new_canvas.Name = "cvsDrawing"
                AddHandler new_canvas.MouseDown, AddressOf cvsDrawing_MouseDown
                AddHandler new_canvas.MouseMove, AddressOf cvsDrawing_MouseMove
                AddHandler new_canvas.MouseUp, AddressOf cvsDrawing_MouseUp

                cvsDrawing = new_canvas
                borDrawing.Child = new_canvas

                borDrawing.Visibility = Windows.Visibility.Visible
                m_FileName = dlg.FileName
            Catch ex As Exception
                MessageBox.Show(ex.Message, _
                    "Error Reading File", _
                    MessageBoxButton.OK, _
                    MessageBoxImage.Error)
            Finally
                fs.Close()
            End Try
        End If
    End Sub

    ' Return True if we can perform the Save action.
    Private Sub DocumentSaveAllowed(ByVal sender As Object, ByVal e As CanExecuteRoutedEventArgs)
        e.CanExecute = ((m_FileName IsNot Nothing) AndAlso m_IsDirty)
    End Sub

    ' Perform the Save action.
    Private Sub DocumentSave(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' See if we have a file name.
        If m_FileName Is Nothing Then
            ' Invoke the Save As command.
            ApplicationCommands.SaveAs.Execute(Nothing, Me)
        End If

        ' Save.
        Dim fs As FileStream = OpenWrite(m_FileName)
        Try
            ' Save.
            XamlWriter.Save(cvsDrawing, fs)

            ' There are no unsaved changes.
            m_IsDirty = False
        Catch ex As Exception
            MessageBox.Show(ex.Message, _
                "Error Saving File", _
                MessageBoxButton.OK, _
                MessageBoxImage.Error)
        Finally
            fs.Close()
        End Try
    End Sub

    ' Return True if we can perform the SaveAs action.
    Private Sub DocumentSaveAsAllowed(ByVal sender As Object, ByVal e As CanExecuteRoutedEventArgs)
        e.CanExecute = (m_IsDirty)
    End Sub

    ' Perform the SaveAs action.
    Private Sub DocumentSaveAs(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' Get the file name.
        Dim dlg As New SaveFileDialog()
        dlg.DefaultExt = ".rects"
        dlg.Filter = "Rectangle Files (.rects)|*.rects|All Files (*.*)|*.*"

        If dlg.ShowDialog() = True Then
            ' Save the selected file name.
            m_FileName = dlg.FileName

            ' Invoke the Save command.
            ApplicationCommands.Save.Execute(Nothing, Me)
        End If
    End Sub

    ' Return True if we can perform the Close action.
    Private Sub DocumentCloseAllowed(ByVal sender As Object, ByVal e As CanExecuteRoutedEventArgs)
        If cvsDrawing Is Nothing Then
            e.CanExecute = False
        Else
            ' Allow if there is a file open.
            e.CanExecute = (borDrawing.Visibility = Windows.Visibility.Visible)
        End If
    End Sub

    ' Perform the Close action.
    Private Sub DocumentClose(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
        ' Make sure it's safe to close.
        If m_IsDirty Then
            If MessageBox.Show("You have unsaved changes. Are you sure you want to close this document?", _
                "Unsaved Changes", MessageBoxButton.YesNo, MessageBoxImage.Exclamation) _
                    = MessageBoxResult.No Then Exit Sub
        End If

        ' Close it.
        borDrawing.Visibility = Windows.Visibility.Hidden
        m_FileName = Nothing
        m_IsDirty = False
    End Sub

#Region "Drawing Commands"
    Private m_NewRectangle As Rectangle = Nothing
    Private m_StartPoint As Point = Nothing

    Private Sub cvsDrawing_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs)
        If (m_NewRectangle IsNot Nothing) Then Exit Sub

        m_StartPoint = Mouse.GetPosition(sender)
        m_NewRectangle = New Rectangle() With {.Width = 0, .Height = 0}
        Canvas.SetLeft(m_NewRectangle, m_StartPoint.X)
        Canvas.SetTop(m_NewRectangle, m_StartPoint.Y)
        
        ' Pick a random color.
        Dim rand As New Random()
        m_NewRectangle.Fill = _
            New SolidColorBrush(Color.FromArgb( _
                255, _
                rand.Next(256), _
                rand.Next(256), _
                rand.Next(256)))
        m_NewRectangle.Stroke = Brushes.Black

        ' Add the Rectangle to the Canvas.
        cvsDrawing.Children.Add(m_NewRectangle)

        ' Remember we've modified the file.
        m_IsDirty = True
    End Sub

    Private Sub cvsDrawing_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Input.MouseEventArgs)
        If (m_NewRectangle Is Nothing) Then Exit Sub

        Dim pt As Point = Mouse.GetPosition(sender)

        Canvas.SetLeft(m_NewRectangle, Math.Min(m_StartPoint.X, pt.X))
        Canvas.SetTop(m_NewRectangle, Math.Min(m_StartPoint.Y, pt.Y))
        m_NewRectangle.Width = Math.Abs(pt.X - m_StartPoint.X)
        m_NewRectangle.Height = Math.Abs(pt.Y - m_StartPoint.Y)
    End Sub

    Private Sub cvsDrawing_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs)
        If (m_NewRectangle Is Nothing) Then Exit Sub
        m_NewRectangle = Nothing
    End Sub
#End Region ' Drawing Commands

    ' Check for unsaved changes.
    Private Sub Window1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If m_IsDirty Then
            e.Cancel = _
                MessageBox.Show("You have unsaved changes. Are you sure you want to quit?", _
                    "Unsaved Changes", MessageBoxButton.YesNo, MessageBoxImage.Exclamation) _
                = MessageBoxResult.No
        End If
    End Sub
End Class
