﻿Imports System.Windows.Threading

Partial Public Class MainPage
    Inherits UserControl

    Public Sub New()
        InitializeComponent()
    End Sub

    Private LastUpdate As DateTime
    Private Balls As List(Of Ball)
    Private WithEvents Clock As DispatcherTimer

    ' Get ready to run.
    Private Sub canField_SizeChanged(ByVal sender As System.Object, ByVal e As System.Windows.SizeChangedEventArgs)
        GetReady()
    End Sub

    ' Get ready to run.
    Private Sub GetReady()
        ' Delete any old Balls. Necessary if the user resizes.
        canField.Children.Clear()

        ' Make some Balls.
        Balls = New List(Of Ball)()
        Dim rand As New Random()
        Dim bounds As New Rect(0, 0, canField.ActualWidth, canField.ActualHeight)
        For i As Integer = 1 To 5
            Balls.Add(RandomBall(rand, bounds, canField))
        Next i

        ' Prepare the timer.
        LastUpdate = DateTime.Now
        Clock = New DispatcherTimer()
        Clock.Interval = TimeSpan.FromSeconds(0.1)
        Clock.Start()
    End Sub

    ' Return a random Ball.
    Private Function RandomBall(ByVal rand As Random, ByVal bounds As Rect, ByVal container As Panel) As Ball
        ' Make the ellipse.
        Dim new_ellipse As New Ellipse()
        container.Children.Add(new_ellipse)
        new_ellipse.Width = rand.Next(50, 150)
        new_ellipse.Height = new_ellipse.Width

        Dim stop1, stop2 As GradientStop
        Dim stops As GradientStopCollection

        ' Make the stroke brush.
        stop1 = New GradientStop()
        stop1.Color = Colors.Green
        stop1.Offset = 0
        stop2 = New GradientStop()
        stop2.Color = Color.FromArgb(255, 0, 255, 0)
        stop2.Offset = 1
        stops = New GradientStopCollection() From {stop1, stop2}
        new_ellipse.Stroke = New LinearGradientBrush(stops, 45)
        new_ellipse.StrokeThickness = 10

        ' Make the fill brush.
        stop1 = New GradientStop()
        stop1.Color = Colors.Green
        stop1.Offset = 1
        stop2 = New GradientStop()
        stop2.Color = Color.FromArgb(255, 0, 255, 0)
        stop2.Offset = 0
        stops = New GradientStopCollection() From {stop1, stop2}
        new_ellipse.Fill = New LinearGradientBrush(stops, 45)

        ' Make the initial transform.
        Dim transform = New TranslateTransform()
        transform.X = rand.Next(0, bounds.Right - new_ellipse.Width)
        transform.Y = rand.Next(0, bounds.Bottom - new_ellipse.Height)
        new_ellipse.RenderTransform = transform

        ' Get velocities.
        Dim vx As Double = rand.Next(30, 100)
        If (rand.Next(0, 2) = 0) Then vx = -vx
        Dim vy As Double = rand.Next(30, 100)
        If (rand.Next(0, 2) = 0) Then vy = -vy

        ' Make the Ball.
        Return New Ball(new_ellipse, transform, vx, vy, bounds)
    End Function

    ' Update the balls.
    Private Sub Clock_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles Clock.Tick
        Dim time_now As DateTime = DateTime.Now
        Dim elapsed As TimeSpan = time_now.Subtract(LastUpdate)

        For Each a_ball As Ball In Balls
            a_ball.Update(elapsed)
        Next a_ball

        LastUpdate = time_now
    End Sub
End Class